@@ -57,10 +57,10 @@ func TestGetEtcdCertVolumes(t *testing.T) {
 			volMount: []v1.VolumeMount{},
 		},
 		{
-			name:     "Should ignore files in /etc/pki",
-			ca:       "/etc/pki/my-etcd-ca.crt",
-			cert:     "/etc/pki/my-etcd.crt",
-			key:      "/etc/pki/my-etcd.key",
+			name:     "Should ignore files in /etc/pki/ca-trust",
+			ca:       "/etc/pki/ca-trust/my-etcd-ca.crt",
+			cert:     "/etc/pki/ca-trust/my-etcd.crt",
+			key:      "/etc/pki/ca-trust/my-etcd.key",
 			vol:      []v1.Volume{},
 			volMount: []v1.VolumeMount{},
 		},
@@ -519,8 +519,9 @@ func TestGetHostPathVolumesForTheControlPlane(t *testing.T) {
 	defer os.RemoveAll(tmpdir)
 
 	// set up tmp caCertsExtraVolumePaths for testing
-	caCertsExtraVolumePaths = []string{fmt.Sprintf("%s/etc/pki", tmpdir), fmt.Sprintf("%s/usr/share/ca-certificates", tmpdir)}
-	defer func() { caCertsExtraVolumePaths = []string{"/etc/pki", "/usr/share/ca-certificates"} }()
+	originalCACertsExtraVolumePaths := caCertsExtraVolumePaths
+	caCertsExtraVolumePaths = []string{fmt.Sprintf("%s/etc/pki/ca-trust", tmpdir), fmt.Sprintf("%s/usr/share/ca-certificates", tmpdir)}
+	defer func() { caCertsExtraVolumePaths = originalCACertsExtraVolumePaths }()
 
 	for _, rt := range tests {
 		t.Run(rt.name, func(t *testing.T) {
@@ -631,7 +632,7 @@ func TestAddExtraHostPathMounts(t *testing.T) {
 			if _, ok := mounts.volumeMounts["component"][volumeName]; !ok {
 				t.Errorf("Expected to find volume mount %q", volumeName)
 			}
-			if *vol.HostPath.Type != v1.HostPathType(hostMount.PathType) {
+			if *vol.HostPath.Type != hostMount.PathType {
 				t.Errorf("Expected to host path type %q", hostMount.PathType)
 			}
 			volMount := mounts.volumeMounts["component"][volumeName]
