@@ -32,8 +32,11 @@ e2e_test=$(kube::util::find-binary "e2e.test")
 # --- Setup some env vars.
 
 GINKGO_PARALLEL=${GINKGO_PARALLEL:-n} # set to 'y' to run tests in parallel
+GINKGO_SILENCE_SKIPS=${GINKGO_SILENCE_SKIPS:-y} # set to 'n' to see S character for each skipped test
+GINKGO_FORCE_NEWLINES=${GINKGO_FORCE_NEWLINES:-$( if [ "${CI:-false}" = "true" ]; then echo "y"; else echo "n"; fi )} # set to 'y' to print a newline after each S or o character
 CLOUD_CONFIG=${CLOUD_CONFIG:-""}
 
+
 # If 'y', Ginkgo's reporter will not use escape sequence to color output.
 #
 # Since Kubernetes 1.25, the default is to use colors only when connected to
@@ -158,6 +161,14 @@ if [[ "${GINKGO_TOLERATE_FLAKES}" == "y" ]]; then
 fi
 ginkgo_args+=("--flake-attempts=${FLAKE_ATTEMPTS}")
 
+if [[ "${GINKGO_SILENCE_SKIPS}" == "y" ]]; then
+  ginkgo_args+=("--silence-skips")
+fi
+
+if [[ "${GINKGO_FORCE_NEWLINES}" == "y" ]]; then
+  ginkgo_args+=("--force-newlines")
+fi
+
 if [[ "${GINKGO_NO_COLOR}" == "y" ]]; then
   ginkgo_args+=("--no-color")
 fi
