@@ -21,7 +21,6 @@ import (
 	"time"
 
 	"github.com/robfig/cron/v3"
-	"k8s.io/utils/pointer"
 
 	batchv1 "k8s.io/api/batch/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -32,6 +31,7 @@ import (
 	"k8s.io/client-go/tools/record"
 	"k8s.io/klog/v2"
 	"k8s.io/kubernetes/pkg/features"
+	"k8s.io/utils/ptr"
 )
 
 // Utilities for dealing with Jobs and CronJobs and time.
@@ -251,7 +251,7 @@ func getJobFromTemplate2(cj *batchv1.CronJob, scheduledTime time.Time) (*batchv1
 
 	if utilfeature.DefaultFeatureGate.Enabled(features.CronJobsScheduledAnnotation) {
 
-		timeZoneLocation, err := time.LoadLocation(pointer.StringDeref(cj.Spec.TimeZone, ""))
+		timeZoneLocation, err := time.LoadLocation(ptr.Deref(cj.Spec.TimeZone, ""))
 		if err != nil {
 			return nil, err
 		}
@@ -277,31 +277,6 @@ func getTimeHashInMinutes(scheduledTime time.Time) int64 {
 	return scheduledTime.Unix() / 60
 }
 
-func getFinishedStatus(j *batchv1.Job) (bool, batchv1.JobConditionType) {
-	for _, c := range j.Status.Conditions {
-		if (c.Type == batchv1.JobComplete || c.Type == batchv1.JobFailed) && c.Status == corev1.ConditionTrue {
-			return true, c.Type
-		}
-	}
-	return false, ""
-}
-
-// IsJobFinished returns whether or not a job has completed successfully or failed.
-func IsJobFinished(j *batchv1.Job) bool {
-	isFinished, _ := getFinishedStatus(j)
-	return isFinished
-}
-
-// IsJobSucceeded returns whether a job has completed successfully.
-func IsJobSucceeded(j *batchv1.Job) bool {
-	for _, c := range j.Status.Conditions {
-		if c.Type == batchv1.JobComplete && c.Status == corev1.ConditionTrue {
-			return true
-		}
-	}
-	return false
-}
-
 // byJobStartTime sorts a list of jobs by start timestamp, using their names as a tie breaker.
 type byJobStartTime []*batchv1.Job
 
