@@ -56,7 +56,7 @@ const (
 
 var (
 	scheme = runtime.NewScheme()
-	codecs = serializer.NewCodecFactory(scheme)
+	codecs = serializer.NewCodecFactory(scheme, serializer.EnableStrict)
 
 	apiVersions = map[string]schema.GroupVersion{
 		credentialproviderv1alpha1.SchemeGroupVersion.String(): credentialproviderv1alpha1.SchemeGroupVersion,
@@ -98,9 +98,11 @@ func RegisterCredentialProviderPlugins(pluginConfigFile, pluginBinDir string) er
 	registerMetrics()
 
 	for _, provider := range credentialProviderConfig.Providers {
-		pluginBin := filepath.Join(pluginBinDir, provider.Name)
-		if _, err := os.Stat(pluginBin); err != nil {
-			if os.IsNotExist(err) {
+		// Considering Windows binary with suffix ".exe", LookPath() helps to find the correct path.
+		// LookPath() also calls os.Stat().
+		pluginBin, err := exec.LookPath(filepath.Join(pluginBinDir, provider.Name))
+		if err != nil {
+			if errors.Is(err, os.ErrNotExist) || errors.Is(err, exec.ErrNotFound) {
 				return fmt.Errorf("plugin binary executable %s did not exist", pluginBin)
 			}
 
@@ -410,7 +412,7 @@ func (e *execPlugin) ExecPlugin(ctx context.Context, image string) (*credentialp
 	cmd.Env = mergeEnvVars(e.environ(), configEnvVars)
 
 	if err = e.runPlugin(ctx, cmd, image); err != nil {
-		return nil, err
+		return nil, fmt.Errorf("%w: %s", err, stderr.String())
 	}
 
 	data = stdout.Bytes()
@@ -494,8 +496,6 @@ func parseRegistry(image string) string {
 // env vars
 func mergeEnvVars(sysEnvVars, credProviderVars []string) []string {
 	mergedEnvVars := sysEnvVars
-	for _, credProviderVar := range credProviderVars {
-		mergedEnvVars = append(mergedEnvVars, credProviderVar)
-	}
+	mergedEnvVars = append(mergedEnvVars, credProviderVars...)
 	return mergedEnvVars
 }
