@@ -56,7 +56,7 @@ func newStorage(t *testing.T) (*etcd3testing.EtcdTestServer, ipallocator.Interfa
 	if err != nil {
 		t.Fatalf("unexpected error creating etcd: %v", err)
 	}
-	s, d, err := generic.NewRawStorage(configForAllocations, nil)
+	s, d, err := generic.NewRawStorage(configForAllocations, nil, nil, "")
 	if err != nil {
 		t.Fatalf("Couldn't create storage: %v", err)
 	}
