@@ -19,6 +19,7 @@ package library
 import (
 	"context"
 	"fmt"
+	"github.com/google/cel-go/common/types/ref"
 	"testing"
 
 	"github.com/google/cel-go/cel"
@@ -30,6 +31,7 @@ import (
 	exprpb "google.golang.org/genproto/googleapis/api/expr/v1alpha1"
 
 	"k8s.io/apiserver/pkg/authorization/authorizer"
+	apiservercel "k8s.io/apiserver/pkg/cel"
 )
 
 const (
@@ -206,6 +208,16 @@ func TestURLsCost(t *testing.T) {
 			expectEsimatedCost: checker.CostEstimate{Min: 4, Max: 4},
 			expectRuntimeCost:  4,
 		},
+		{
+			ops:                []string{" == url('https:://kubernetes.io/')"},
+			expectEsimatedCost: checker.CostEstimate{Min: 7, Max: 9},
+			expectRuntimeCost:  7,
+		},
+		{
+			ops:                []string{" == url('http://x.b')"},
+			expectEsimatedCost: checker.CostEstimate{Min: 5, Max: 5},
+			expectRuntimeCost:  5,
+		},
 	}
 
 	for _, tc := range cases {
@@ -245,6 +257,14 @@ func TestIPCost(t *testing.T) {
 			},
 			expectRuntimeCost: func(c uint64) uint64 { return c + 1 },
 		},
+		{
+			ops: []string{" == ip('192.168.0.1')"},
+			// For most other operations, the cost is expected to be the base + 1.
+			expectEsimatedCost: func(c checker.CostEstimate) checker.CostEstimate {
+				return c.Add(ipv4BaseEstimatedCost).Add(checker.CostEstimate{Min: 1, Max: 1})
+			},
+			expectRuntimeCost: func(c uint64) uint64 { return c + ipv4BaseRuntimeCost + 1 },
+		},
 	}
 
 	for _, tc := range testCases {
@@ -320,6 +340,14 @@ func TestCIDRCost(t *testing.T) {
 			},
 			expectRuntimeCost: func(c uint64) uint64 { return c + 1 },
 		},
+		{
+			ops: []string{" == cidr('2001:db8::/32')"},
+			// For most other operations, the cost is expected to be the base + 1.
+			expectEsimatedCost: func(c checker.CostEstimate) checker.CostEstimate {
+				return c.Add(ipv6BaseEstimatedCost).Add(checker.CostEstimate{Min: 1, Max: 1})
+			},
+			expectRuntimeCost: func(c uint64) uint64 { return c + ipv6BaseRuntimeCost + 1 },
+		},
 	}
 
 	//nolint:gocritic
@@ -601,6 +629,12 @@ func TestStringLibrary(t *testing.T) {
 			expectEsimatedCost: checker.CostEstimate{Min: 2, Max: 2},
 			expectRuntimeCost:  2,
 		},
+		{
+			name:               "jsonpatch.escapeKey",
+			expr:               "jsonpatch.escapeKey('abc/def~ abc/def~')",
+			expectEsimatedCost: checker.CostEstimate{Min: 2, Max: 2},
+			expectRuntimeCost:  2,
+		},
 	}
 
 	for _, tc := range cases {
@@ -708,19 +742,19 @@ func TestQuantityCost(t *testing.T) {
 		{
 			name:                "equality_reflexivity",
 			expr:                `quantity("200M") == quantity("200M")`,
-			expectEstimatedCost: checker.CostEstimate{Min: 3, Max: 1844674407370955266},
+			expectEstimatedCost: checker.CostEstimate{Min: 3, Max: 3},
 			expectRuntimeCost:   3,
 		},
 		{
 			name:                "equality_symmetry",
 			expr:                `quantity("200M") == quantity("0.2G") && quantity("0.2G") == quantity("200M")`,
-			expectEstimatedCost: checker.CostEstimate{Min: 3, Max: 3689348814741910532},
+			expectEstimatedCost: checker.CostEstimate{Min: 3, Max: 6},
 			expectRuntimeCost:   6,
 		},
 		{
 			name:                "equality_transitivity",
 			expr:                `quantity("2M") == quantity("0.002G") && quantity("2000k") == quantity("2M") && quantity("0.002G") == quantity("2000k")`,
-			expectEstimatedCost: checker.CostEstimate{Min: 3, Max: 5534023222112865798},
+			expectEstimatedCost: checker.CostEstimate{Min: 3, Max: 9},
 			expectRuntimeCost:   9,
 		},
 		{
@@ -744,19 +778,19 @@ func TestQuantityCost(t *testing.T) {
 		{
 			name:                "add_quantity",
 			expr:                `quantity("50k").add(quantity("20")) == quantity("50.02k")`,
-			expectEstimatedCost: checker.CostEstimate{Min: 5, Max: 1844674407370955268},
+			expectEstimatedCost: checker.CostEstimate{Min: 5, Max: 5},
 			expectRuntimeCost:   5,
 		},
 		{
 			name:                "sub_quantity",
 			expr:                `quantity("50k").sub(quantity("20")) == quantity("49.98k")`,
-			expectEstimatedCost: checker.CostEstimate{Min: 5, Max: 1844674407370955268},
+			expectEstimatedCost: checker.CostEstimate{Min: 5, Max: 5},
 			expectRuntimeCost:   5,
 		},
 		{
 			name:                "sub_int",
 			expr:                `quantity("50k").sub(20) == quantity("49980")`,
-			expectEstimatedCost: checker.CostEstimate{Min: 4, Max: 1844674407370955267},
+			expectEstimatedCost: checker.CostEstimate{Min: 4, Max: 4},
 			expectRuntimeCost:   4,
 		},
 		{
@@ -825,6 +859,18 @@ func TestNameFormatCost(t *testing.T) {
 			expectEstimatedCost: checker.CostEstimate{Min: 34, Max: 34},
 			expectRuntimeCost:   10,
 		},
+		{
+			name:                "format.dns1123label.validate",
+			expr:                `format.named("dns1123Label").value().validate("my-name")`,
+			expectEstimatedCost: checker.CostEstimate{Min: 34, Max: 34},
+			expectRuntimeCost:   10,
+		},
+		{
+			name:                "format.dns1123label.validate",
+			expr:                `format.named("dns1123Label").value() == format.named("dns1123Label").value()`,
+			expectEstimatedCost: checker.CostEstimate{Min: 5, Max: 11},
+			expectRuntimeCost:   5,
+		},
 	}
 
 	for _, tc := range cases {
@@ -1064,6 +1110,95 @@ func TestSetsCost(t *testing.T) {
 	}
 }
 
+func TestTwoVariableComprehensionCost(t *testing.T) {
+	cases := []struct {
+		name                string
+		expr                string
+		expectEstimatedCost checker.CostEstimate
+		expectRuntimeCost   uint64
+	}{
+		{
+			name:                "map all",
+			expr:                `{'a': 1, 'b': 2}.all(k, v, v > 0)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 37, Max: 41},
+			expectRuntimeCost:   41,
+		},
+		{
+			name:                "map exists",
+			expr:                `{'a': 1, 'b': 2}.exists(k, v, v > 0)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 39, Max: 43},
+			expectRuntimeCost:   40,
+		},
+		{
+			name:                "map existsOne",
+			expr:                `{'a': 1, 'b': 2}.existsOne(k, v, v > 0)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 38, Max: 40},
+			expectRuntimeCost:   40,
+		},
+		{
+			name:                "map transformMap",
+			expr:                `{'a': 1, 'b': 2}.transformMap(k, v, v + 1)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 71, Max: 71},
+			expectRuntimeCost:   71,
+		},
+		{
+			name:                "map transformMap with filter",
+			expr:                `{'a': 1, 'b': 2}.transformMap(k, v, v < 5, v + 1)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 67, Max: 75},
+			expectRuntimeCost:   75,
+		},
+		{
+			name:                "map transformMapEntry",
+			expr:                `{'a': 1, 'b': 2}.transformMapEntry(k, v, {k: v + 1})`,
+			expectEstimatedCost: checker.CostEstimate{Min: 131, Max: 131},
+			expectRuntimeCost:   131,
+		},
+		{
+			name:                "map transformMapEntry with filter",
+			expr:                `{'a': 1, 'b': 2}.transformMapEntry(k, v, v < 5, {k: v + 1})`,
+			expectEstimatedCost: checker.CostEstimate{Min: 67, Max: 135},
+			expectRuntimeCost:   135,
+		},
+
+		{
+			name:                "list all",
+			expr:                `[1, 2].all(i, v, v > 0)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 17, Max: 21},
+			expectRuntimeCost:   21,
+		},
+		{
+			name:                "list exists",
+			expr:                `[1, 2].exists(i, v, v > 0)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 19, Max: 23},
+			expectRuntimeCost:   20,
+		},
+		{
+			name:                "list existsOne",
+			expr:                `[1, 2].existsOne(i, v, v > 0)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 18, Max: 20},
+			expectRuntimeCost:   20,
+		},
+		{
+			name:                "list transformList",
+			expr:                `[1, 2].transformList(i, v, v + 1)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 49, Max: 49},
+			expectRuntimeCost:   49,
+		},
+		{
+			name:                "list transformList with filter",
+			expr:                `[1, 2].transformList(i, v, v < 5, v + 1)`,
+			expectEstimatedCost: checker.CostEstimate{Min: 27, Max: 53},
+			expectRuntimeCost:   53,
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			testCost(t, tc.expr, tc.expectEstimatedCost, tc.expectRuntimeCost)
+		})
+	}
+}
+
 func testCost(t *testing.T, expr string, expectEsimatedCost checker.CostEstimate, expectRuntimeCost uint64) {
 	originalPanicOnUnknown := panicOnUnknown
 	panicOnUnknown = true
@@ -1082,10 +1217,12 @@ func testCost(t *testing.T, expr string, expectEsimatedCost checker.CostEstimate
 		IP(),
 		CIDR(),
 		Format(),
+		JSONPatch(),
 		cel.OptionalTypes(),
 		// cel-go v0.17.7 introduced CostEstimatorOptions.
 		// Previous the presence has a cost of 0 but cel fixed it to 1. We still set to 0 here to avoid breaking changes.
 		cel.CostEstimatorOptions(checker.PresenceTestHasCost(false)),
+		ext.TwoVarComprehensions(),
 	)
 	if err != nil {
 		t.Fatalf("%v", err)
@@ -1193,10 +1330,10 @@ func TestSize(t *testing.T) {
 	est := &CostEstimator{SizeEstimator: &testCostEstimator{}}
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
-			var targetNode checker.AstNode = testSizeNode{size: tc.targetSize}
+			var targetNode checker.AstNode = testNode{size: tc.targetSize}
 			argNodes := make([]checker.AstNode, len(tc.argSizes))
 			for i, arg := range tc.argSizes {
-				argNodes[i] = testSizeNode{size: arg}
+				argNodes[i] = testNode{size: arg}
 			}
 			result := est.EstimateCallCost(tc.function, tc.overload, &targetNode, argNodes)
 			if result.ResultSize == nil {
@@ -1209,25 +1346,63 @@ func TestSize(t *testing.T) {
 	}
 }
 
-type testSizeNode struct {
+// TestTypeEquality ensures that cost is tested for all custom types used by Kubernetes libraries.
+func TestTypeEquality(t *testing.T) {
+	examples := map[string]ref.Val{
+		// Add example ref.Val's for custom types in Kubernetes here:
+		"kubernetes.authorization.Authorizer":    authorizerVal{},
+		"kubernetes.authorization.PathCheck":     pathCheckVal{},
+		"kubernetes.authorization.GroupCheck":    groupCheckVal{},
+		"kubernetes.authorization.ResourceCheck": resourceCheckVal{},
+		"kubernetes.authorization.Decision":      decisionVal{},
+		"kubernetes.URL":                         apiservercel.URL{},
+		"kubernetes.Quantity":                    apiservercel.Quantity{},
+		"net.IP":                                 apiservercel.IP{},
+		"net.CIDR":                               apiservercel.CIDR{},
+		"kubernetes.NamedFormat":                 apiservercel.Format{},
+		"kubernetes.Semver":                      apiservercel.Semver{},
+	}
+
+	originalPanicOnUnknown := panicOnUnknown
+	panicOnUnknown = true
+	t.Cleanup(func() { panicOnUnknown = originalPanicOnUnknown })
+	est := &CostEstimator{SizeEstimator: &testCostEstimator{}}
+
+	for _, lib := range KnownLibraries() {
+		for _, kt := range lib.Types() {
+			t.Run(kt.TypeName(), func(t *testing.T) {
+				typeNode := testNode{size: checker.SizeEstimate{Min: 10, Max: 100}, typ: kt}
+				est.EstimateCallCost("_==_", "", nil, []checker.AstNode{typeNode, typeNode})
+				ex, ok := examples[kt.TypeName()]
+				if !ok {
+					t.Errorf("missing example for type: %s", kt.TypeName())
+				}
+				est.CallCost("_==_", "", []ref.Val{ex, ex}, nil)
+			})
+		}
+	}
+}
+
+type testNode struct {
 	size checker.SizeEstimate
+	typ  *types.Type
 }
 
-var _ checker.AstNode = (*testSizeNode)(nil)
+var _ checker.AstNode = (*testNode)(nil)
 
-func (t testSizeNode) Path() []string {
+func (t testNode) Path() []string {
 	return nil // not needed
 }
 
-func (t testSizeNode) Type() *types.Type {
+func (t testNode) Type() *types.Type {
+	return t.typ // not needed
+}
+
+func (t testNode) Expr() ast.Expr {
 	return nil // not needed
 }
 
-func (t testSizeNode) Expr() ast.Expr {
-	return nil // not needed
-}
-
-func (t testSizeNode) ComputedSize() *checker.SizeEstimate {
+func (t testNode) ComputedSize() *checker.SizeEstimate {
 	return &t.size
 }
 
