@@ -243,7 +243,7 @@ run_client_config_tests() {
 
   # test invalid config
   kubectl config view | sed -E "s/apiVersion: .*/apiVersion: v-1/g" > "${TMPDIR:-/tmp}"/newconfig.yaml
-  output_message=$(! "${KUBE_OUTPUT_HOSTBIN}/kubectl" get pods --context="" --user="" --kubeconfig="${TMPDIR:-/tmp}"/newconfig.yaml 2>&1)
+  output_message=$(! "${THIS_PLATFORM_BIN}/kubectl" get pods --context="" --user="" --kubeconfig="${TMPDIR:-/tmp}"/newconfig.yaml 2>&1)
   kube::test::if_has_string "${output_message}" "error loading config file"
 
   output_message=$(! kubectl get pod --kubeconfig=missing-config 2>&1)
@@ -298,4 +298,4 @@ EOF
 
   set +o nounset
   set +o errexit
-}
\ No newline at end of file
+}
