@@ -45,8 +45,10 @@ import (
 	"k8s.io/kubernetes/test/images/agnhost/porter"
 	resconsumerctrl "k8s.io/kubernetes/test/images/agnhost/resource-consumer-controller"
 	servehostname "k8s.io/kubernetes/test/images/agnhost/serve-hostname"
+	tcpreset "k8s.io/kubernetes/test/images/agnhost/tcp-reset"
 	testwebserver "k8s.io/kubernetes/test/images/agnhost/test-webserver"
 	"k8s.io/kubernetes/test/images/agnhost/webhook"
+	"k8s.io/kubernetes/third_party/forked/vishhstress" // MIT License
 )
 
 var Version = "development"
@@ -81,9 +83,11 @@ func main() {
 	rootCmd.AddCommand(resconsumerctrl.CmdResourceConsumerController)
 	rootCmd.AddCommand(servehostname.CmdServeHostname)
 	rootCmd.AddCommand(testwebserver.CmdTestWebserver)
+	rootCmd.AddCommand(tcpreset.CmdTCPReset)
 	rootCmd.AddCommand(webhook.CmdWebhook)
 	rootCmd.AddCommand(openidmetadata.CmdTestServiceAccountIssuerDiscovery)
 	rootCmd.AddCommand(grpchealthchecking.CmdGrpcHealthChecking)
+	rootCmd.AddCommand(vishhstress.CmdStress)
 
 	// NOTE(claudiub): Some tests are passing logging related flags, so we need to be able to
 	// accept them. This will also include them in the printed help.
