@@ -23,14 +23,12 @@ import (
 	"time"
 
 	v1 "k8s.io/api/core/v1"
-	networkingv1alpha1 "k8s.io/api/networking/v1alpha1"
+	networkingv1beta1 "k8s.io/api/networking/v1beta1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/client-go/informers"
 	clientset "k8s.io/client-go/kubernetes"
-	featuregatetesting "k8s.io/component-base/featuregate/testing"
 	kubeapiservertesting "k8s.io/kubernetes/cmd/kube-apiserver/app/testing"
 	"k8s.io/kubernetes/pkg/controller/servicecidrs"
 	"k8s.io/kubernetes/pkg/controlplane/controller/defaultservicecidr"
@@ -49,7 +47,6 @@ import (
 // 6. start the new apiserver with the new ServiceCIDRs on the flags and shutdown the old one
 // 7. delete the kubernetes.default service, the new apiserver will recreate it within the new ServiceCIDR
 func TestMigrateServiceCIDR(t *testing.T) {
-	defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.MultiCIDRServiceAllocator, true)()
 	tCtx := ktesting.Init(t)
 
 	cidr1 := "192.168.0.0/29"
@@ -60,10 +57,11 @@ func TestMigrateServiceCIDR(t *testing.T) {
 	s1 := kubeapiservertesting.StartTestServerOrDie(t,
 		apiServerOptions,
 		[]string{
-			"--runtime-config=networking.k8s.io/v1alpha1=true",
+			"--runtime-config=networking.k8s.io/v1beta1=true",
 			"--service-cluster-ip-range=" + cidr1,
 			"--advertise-address=10.1.1.1",
 			"--disable-admission-plugins=ServiceAccount",
+			fmt.Sprintf("--feature-gates=%s=true,%s=true", features.MultiCIDRServiceAllocator, features.DisableAllocatorDualWrite),
 		},
 		etcdOptions)
 
@@ -79,15 +77,16 @@ func TestMigrateServiceCIDR(t *testing.T) {
 	// ServiceCIDR controller
 	go servicecidrs.NewController(
 		tCtx,
-		informers1.Networking().V1alpha1().ServiceCIDRs(),
-		informers1.Networking().V1alpha1().IPAddresses(),
+		informers1.Networking().V1beta1().ServiceCIDRs(),
+		informers1.Networking().V1beta1().IPAddresses(),
 		client1,
 	).Run(tCtx, 5)
 	informers1.Start(tCtx.Done())
+	informers1.WaitForCacheSync(tCtx.Done())
 
 	// the default serviceCIDR should have a finalizer and ready condition set to true
 	if err := wait.PollUntilContextTimeout(context.Background(), 1*time.Second, time.Minute, false, func(ctx context.Context) (bool, error) {
-		cidr, err := client1.NetworkingV1alpha1().ServiceCIDRs().Get(context.TODO(), defaultservicecidr.DefaultServiceCIDRName, metav1.GetOptions{})
+		cidr, err := client1.NetworkingV1beta1().ServiceCIDRs().Get(context.TODO(), defaultservicecidr.DefaultServiceCIDRName, metav1.GetOptions{})
 		if err != nil && !apierrors.IsNotFound(err) {
 			return false, err
 		}
@@ -121,13 +120,13 @@ func TestMigrateServiceCIDR(t *testing.T) {
 		}
 	}
 	// Add a new service CIDR to be able to migrate the apiserver
-	if _, err := client1.NetworkingV1alpha1().ServiceCIDRs().Create(context.Background(), makeServiceCIDR("migration-cidr", cidr2, ""), metav1.CreateOptions{}); err != nil {
+	if _, err := client1.NetworkingV1beta1().ServiceCIDRs().Create(context.Background(), makeServiceCIDR("migration-cidr", cidr2, ""), metav1.CreateOptions{}); err != nil {
 		t.Fatalf("got unexpected error: %v", err)
 	}
 
 	// wait ServiceCIDR is ready
 	if err := wait.PollUntilContextTimeout(context.Background(), 1*time.Second, time.Minute, false, func(ctx context.Context) (bool, error) {
-		cidr, err := client1.NetworkingV1alpha1().ServiceCIDRs().Get(context.TODO(), "migration-cidr", metav1.GetOptions{})
+		cidr, err := client1.NetworkingV1beta1().ServiceCIDRs().Get(context.TODO(), "migration-cidr", metav1.GetOptions{})
 		if err != nil && !apierrors.IsNotFound(err) {
 			return false, err
 		}
@@ -137,18 +136,18 @@ func TestMigrateServiceCIDR(t *testing.T) {
 	}
 
 	// delete the default ServiceCIDR so is no longer used for allocating IPs
-	if err := client1.NetworkingV1alpha1().ServiceCIDRs().Delete(context.Background(), defaultservicecidr.DefaultServiceCIDRName, metav1.DeleteOptions{}); err != nil {
+	if err := client1.NetworkingV1beta1().ServiceCIDRs().Delete(context.Background(), defaultservicecidr.DefaultServiceCIDRName, metav1.DeleteOptions{}); err != nil {
 		t.Fatalf("got unexpected error: %v", err)
 	}
 
 	// the default serviceCIDR should be pending deletion with Ready condition set to false
 	if err := wait.PollUntilContextTimeout(context.Background(), 1*time.Second, time.Minute, false, func(ctx context.Context) (bool, error) {
-		cidr, err := client1.NetworkingV1alpha1().ServiceCIDRs().Get(context.TODO(), defaultservicecidr.DefaultServiceCIDRName, metav1.GetOptions{})
+		cidr, err := client1.NetworkingV1beta1().ServiceCIDRs().Get(context.TODO(), defaultservicecidr.DefaultServiceCIDRName, metav1.GetOptions{})
 		if err != nil && !apierrors.IsNotFound(err) {
 			return false, err
 		}
 		for _, condition := range cidr.Status.Conditions {
-			if condition.Type == networkingv1alpha1.ServiceCIDRConditionReady {
+			if condition.Type == networkingv1beta1.ServiceCIDRConditionReady {
 				return condition.Status == metav1.ConditionFalse, nil
 			}
 		}
@@ -168,18 +167,22 @@ func TestMigrateServiceCIDR(t *testing.T) {
 		if svc.Name == "kubernetes" {
 			continue
 		}
+
+		if err := client1.CoreV1().Services(svc.Namespace).Delete(context.Background(), svc.Name, metav1.DeleteOptions{}); err != nil {
+			t.Fatalf("got unexpected error: %v", err)
+		}
+		t.Logf("Deleted Service with IP %s", svc.Spec.ClusterIP)
+
 		// wipe the necessary fields so we can recreate the Service
 		svc.ResourceVersion = ""
 		svc.Spec.ClusterIP = ""
 		svc.Spec.ClusterIPs = nil
 		svc.Status = v1.ServiceStatus{}
-		if err := client1.CoreV1().Services(svc.Namespace).Delete(context.Background(), svc.Name, metav1.DeleteOptions{}); err != nil {
-			t.Fatalf("got unexpected error: %v", err)
-		}
 		svc, err := client1.CoreV1().Services(svc.Namespace).Create(context.Background(), &svc, metav1.CreateOptions{})
 		if err != nil {
 			t.Fatalf("got unexpected error: %v", err)
 		}
+		t.Logf("Created Service with IP %s", svc.Spec.ClusterIP)
 		if !cidrContainsIP(cidr2, svc.Spec.ClusterIP) {
 			t.Fatalf("Service expected to have an ip in range 10.168.0.0/24, got %s", svc.Spec.ClusterIP)
 		}
@@ -189,7 +192,7 @@ func TestMigrateServiceCIDR(t *testing.T) {
 	s2 := kubeapiservertesting.StartTestServerOrDie(t,
 		apiServerOptions,
 		[]string{
-			"--runtime-config=networking.k8s.io/v1alpha1=true",
+			"--runtime-config=networking.k8s.io/v1beta1=true",
 			"--service-cluster-ip-range=" + cidr2,
 			"--advertise-address=10.1.1.1",
 			"--disable-admission-plugins=ServiceAccount",
@@ -209,15 +212,16 @@ func TestMigrateServiceCIDR(t *testing.T) {
 
 	// ServiceCIDR controller
 	tCtx2 := ktesting.Init(t)
-	defer tCtx.Cancel("tearing down ServiceCIDR controller 2")
+	defer tCtx2.Cancel("tearing down ServiceCIDR controller 2")
 	informers2 := informers.NewSharedInformerFactory(client2, resyncPeriod)
 	go servicecidrs.NewController(
 		tCtx2,
-		informers2.Networking().V1alpha1().ServiceCIDRs(),
-		informers2.Networking().V1alpha1().IPAddresses(),
+		informers2.Networking().V1beta1().ServiceCIDRs(),
+		informers2.Networking().V1beta1().IPAddresses(),
 		client2,
 	).Run(tCtx2, 5)
 	informers2.Start(tCtx2.Done())
+	informers2.WaitForCacheSync(tCtx.Done())
 
 	// delete the kubernetes.default service so the old DefaultServiceCIDR can be deleted
 	// and the new apiserver can take over
@@ -227,25 +231,29 @@ func TestMigrateServiceCIDR(t *testing.T) {
 
 	// the default serviceCIDR should  be the new one
 	if err := wait.PollUntilContextTimeout(context.Background(), 1*time.Second, time.Minute, false, func(ctx context.Context) (bool, error) {
-		cidr, err := client2.NetworkingV1alpha1().ServiceCIDRs().Get(context.TODO(), defaultservicecidr.DefaultServiceCIDRName, metav1.GetOptions{})
+		cidr, err := client2.NetworkingV1beta1().ServiceCIDRs().Get(context.TODO(), defaultservicecidr.DefaultServiceCIDRName, metav1.GetOptions{})
 		if err != nil && !apierrors.IsNotFound(err) {
 			return false, err
 		}
 
 		if len(cidr.Spec.CIDRs) == 0 {
+			t.Logf("No CIDR available")
 			return false, nil
 		}
 
 		if cidr.Spec.CIDRs[0] != cidr2 {
+			t.Logf("CIDR expected %s got %s", cidr2, cidr.Spec.CIDRs[0])
 			return false, nil
 		}
 
 		if len(cidr.Finalizers) == 0 {
+			t.Logf("Expected finalizer to be set")
 			return false, nil
 		}
 
 		for _, condition := range cidr.Status.Conditions {
-			if condition.Type == networkingv1alpha1.ServiceCIDRConditionReady {
+			if condition.Type == networkingv1beta1.ServiceCIDRConditionReady {
+				t.Logf("Expected Condition %s to be %s", condition.Status, metav1.ConditionTrue)
 				return condition.Status == metav1.ConditionTrue, nil
 			}
 		}
@@ -269,13 +277,13 @@ func TestMigrateServiceCIDR(t *testing.T) {
 	}
 
 	// The temporary ServiceCIDR can be deleted now since the Default ServiceCIDR will cover it
-	if err := client2.NetworkingV1alpha1().ServiceCIDRs().Delete(context.Background(), "migration-cidr", metav1.DeleteOptions{}); err != nil {
+	if err := client2.NetworkingV1beta1().ServiceCIDRs().Delete(context.Background(), "migration-cidr", metav1.DeleteOptions{}); err != nil {
 		t.Fatalf("got unexpected error: %v", err)
 	}
 
 	// wait ServiceCIDR no longer exist
 	if err := wait.PollUntilContextTimeout(context.Background(), 1*time.Second, time.Minute, false, func(ctx context.Context) (bool, error) {
-		_, err := client2.NetworkingV1alpha1().ServiceCIDRs().Get(context.TODO(), "migration-cidr", metav1.GetOptions{})
+		_, err := client2.NetworkingV1beta1().ServiceCIDRs().Get(context.TODO(), "migration-cidr", metav1.GetOptions{})
 		if err != nil && !apierrors.IsNotFound(err) {
 			return false, nil
 		}
